
/*
 * The Verus Model Checking System
 * Copyright (c) 1997 Carnegie Mellon University
 * All Rights Reserved. See file license.txt for details.
 *
 *
 * Routines for manipulating BDDs: redirects bdd operations to proper
 * routine in the bdd package.
 *
 * Sergio Campos -- 1997
 *
 */

#ifndef BDD_H
#define BDD_H

#include <bdduser.h>
#include <symbols.h>

extern bdd_manager bddm;

#define V_MAX_VARIABLES 300

/*
 * A typed bdd is a bdd with a type associated. The type says if the
 * bdd related to a boolean or an integer value in the program.
 */
typedef struct typedbdd *typedbdd_ptr;

struct typedbdd{
  bdd           tbdd;
  variable_type type;
};


/*
 * Creates bdd variables for verus variables and returns its id.
 * Input: vtype
 */
int v_new_bdd();

/*
 * Frees the space used by a typed bdd.
 */
void v_free_bdd();

void force_gc(); /* forces a garbage collection */


/*
 * BDD stack implementation - Used to store intermediate relations
 *    during compilation.
 */

typedef struct stack *stack_ptr;

struct stack{
  bdd          top;
  stack_ptr   next;
};

vresult Rpush();
bdd Rpop();

/*
 * BDD name handling functions.
 */
void bdd_to_name_insert();
void v_print_bdd();
char *bdd_to_name();

vresult write_ordering_file();
vresult read_ordering_file();

/*
 * Variable association handles
 *
 * They are used in two ways, to existentially quantify out current or
 * next state vars from a bdd, or to translate variables in a bdd from
 * the current (next) state to next (current) state. Both functions are
 * used in the step functions of the model checker.
 *
 * current_assoc: Quantify out current state vars (bdd_exists)
 *                Translate from current state to next state (bdd_substitute)
 * next_assoc:    Quantify out next state vars (bdd_exists)
 *                Translate from next state to current state (bdd_substitute)
 */
extern int current_assoc;
extern int next_assoc;
extern int uservars_assoc;
extern int uservars_last;

void init_bdds();
void init_assoc();
void setup_assocs();

extern bdd current_assoc_array[V_MAX_VARIABLES+1];
extern bdd next_assoc_array[V_MAX_VARIABLES+1];
extern bdd uservars_assoc_array[V_MAX_VARIABLES+1];

/*
 * varname_lastvar is a pointer into the variable reordering file. If it
 * is 0, there is no reordering file.
 */
extern char *varname_lastvar;
int read_order_file_lookup();

#endif

