
/*
 * The Verus Model Checking System
 * Copyright (c) 1997 Carnegie Mellon University
 * All Rights Reserved. See file license.txt for details.
 *
 *
 * Header file for the compiler for the Verus tool.
 *
 * Sergio Campos -- 1997
 */

#ifndef COMPILE_H
#define COMPILE_H

#include <bdd.h>


/*
 * R is the relation being constructed incrementally by the compiler. R 
 * determines the changes caused by the execution of statements. So,
 * given a state x, R(x, y) is true if y can be reached from x by
 * executing the statement that generated R.
 */
extern bdd R;

typedef struct id_list *id_list_ptr;
struct id_list{
  char         *element;
  id_list_ptr  next;     /* position to insert next item */
};

/* Function structure */
typedef struct fn_list *fn_list_ptr;
struct fn_list{
  char        *fname;
  bdd         TR,            /* transition relation   */
              Init;          /* initial state         */
  int         nwaitvars,     /* number of wait vars   */
              nvars,         /* number of variables   */
              waitcount;     /* number of wait()'s    */
  id_list_ptr vars;          /* variables in function */
                    /* TR depends on the next value of these variables    */
  bdd         *depend_next;
                    /* TR depends on the current value of these variables */
  bdd         *depend_current;
  int         next_assoc, current_assoc;
  fn_list_ptr next;
};

extern fn_list_ptr functions;
extern id_list_ptr global_vars;

extern bdd global_identity;

#define WAITCOUNT_NAME    "_wc"

void          init_compile();
id_list_ptr   reduce_identifier_list();
int           reduce_declaration();
typedbdd_ptr  id_to_bdd();
typedbdd_ptr  id_to_bdd_current();
typedbdd_ptr  const_to_bdd();
typedbdd_ptr  assignment_expression();
typedbdd_ptr  reduce_IMPLIES();
typedbdd_ptr  reduce_IFF();
typedbdd_ptr  reduce_OR_expression();
typedbdd_ptr  reduce_AND_expression();
typedbdd_ptr  reduce_equality_expression();
typedbdd_ptr  reduce_relational_expression();
typedbdd_ptr  reduce_additive_expression();
typedbdd_ptr  reduce_multiplicative_expression();
typedbdd_ptr  reduce_unary_expression();
typedbdd_ptr  reduce_expression_list();
int           reduce_assignment();
int           reduce_time_stmt();
int           reduce_function_definition();
int           reduce_then();
int           reduce_else();
int           reduce_if_stmt();
int           reduce_while_condition();
int           reduce_while();
int           reduce_assert();

int           reduce_spec();
typedbdd_ptr  reduce_ctl_expr();
int           reduce_min();
int           reduce_max();
int           reduce_mincount();
int           reduce_maxcount();
int           reduce_example();

void v_print_state();
void v_print_transition();

#endif
