
/*
 * The Verus Model Checking System
 * Copyright (c) 1997 Carnegie Mellon University
 * All Rights Reserved. See file license.txt for details.
 *
 *
 * Integer conversions for the Verus compiler
 *
 * Sergio Campos -- 1997
 */

#include<stdio.h>
#include<strings.h>
#include<stdlib.h>
#include<symbols.h>
#include<bdd.h>
#include<integer.h>


extern assoc_last;

/*
 * Creates bdd variables for a new integer variable
 */
void new_int_var(symbol_name, width, v1, v2, v3)
char   *symbol_name;
int    width;
int    *v1;
int    *v2;
int    *v3;
{
  intvar vv1, vv2, vv3;
  int i, pos;
  char *s;
  char si[4];

  vv3 = malloc((width + 1) * sizeof(int));
  vv2 = malloc((width + 1) * sizeof(int));
  vv1 = malloc((width + 1) * sizeof(int));

  for (i = 0; i < width; i++) {
    s = malloc(strlen(symbol_name) + 15);
    strcpy(s, symbol_name);
    strcat(s, "[");
    sprintf(si, "%i", width-i-1);
    strcat(s, si);
    strcat(s, "]");
    pos = read_order_file_lookup(s);
    free(s);
    if (pos != VNOTFOUND) {

      /* variables have already been created. Find out
         who they are */
      vv2[i] = bdd_var_with_index(bddm, pos); /* current */

      s = malloc(strlen(symbol_name) + 15);
      strcpy(s, symbol_name);
      strcat(s, "'[");
      strcat(s, si);
      strcat(s, "]");
      pos = read_order_file_lookup(s);
      if (pos == VNOTFOUND)
        fatal_error("Missing variable in ordering file: %s", s);
      vv1[i] = bdd_var_with_index(bddm, pos); /* next    */

      strcpy(s, symbol_name);
      strcat(s, "_aux[");
      strcat(s, si);
      strcat(s, "]");
      pos = read_order_file_lookup(s);
      if (pos == VNOTFOUND)
        fatal_error("Missing variable in ordering file: %s", s);
      vv3[i] = bdd_var_with_index(bddm, pos); /* aux     */
      free(s);
    } else {
      if (varname_lastvar != 0)
        error("Creating variable not in order file: %s.", symbol_name);
      vv3[i] = bdd_new_var_last(bddm);
      vv2[i] = bdd_new_var_last(bddm);
      vv1[i] = bdd_new_var_last(bddm);
    };
    if (assoc_last+2 > V_MAX_VARIABLES) {
      fatal_error("Maximum number of variables exceeded.\n");
    };
    current_assoc_array[assoc_last] = vv2[i];
    next_assoc_array[assoc_last]    = vv1[i];
    assoc_last++;
    current_assoc_array[assoc_last] = vv1[i];
    next_assoc_array[assoc_last]    = vv2[i];
    assoc_last++;
  };
  vv3[width] = 0;
  vv2[width] = 0;
  vv1[width] = 0;
  *v1 = (int) vv1;
  *v2 = (int) vv2;
  *v3 = (int) vv3;
};


/*
 * Converts an integer value into its corresponding bdd. The integer
 * variable v has the bdd variables used in the final bdd.
 */
bdd int_to_bdd(value, v)
int value;
intvar v;
{
  bdd temp[2];
  bdd result;
  int i, n, actual_value;

  result = bdd_one(bddm);
  actual_value = 0;

  /* Find last bdd in the variable. This will be the LSB */
  for(n = 0; v[n] != 0; n++) ;
  n--;

  for(i = n; i >= 0; i--) {
    if (value & 0x1) {
      temp[0] = bdd_identity(bddm, v[i]);
      actual_value += 1 << (n - i);
    } else {
      temp[0] = bdd_not(bddm, v[i]);
    };
    temp[1] = result;
    result = bdd_and(bddm, result, temp[0]);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);

    value >>= 1;
  };
  if (value != 0) {
    error("Integer conversion overflow. Value represented is %d.",
                 actual_value);
  };

  return(result);
};


/*
 * Converts a bdd array into its corresponding integer value.
 */
int bdd_to_int(v)
intvar v;
{
  return(0);
};


/*
 * vadd = v1 + v2 modulo MAX
 */
intvar vadd(v1, v2, carryin, context)
intvar v1, v2;
bdd    carryin;
bdd    context;
{
  intvar result;
  int i, n;
  bdd temp[2];
  bdd carry, newcarry;

  /* assumes v1 and v2 have the same length */

  /* Find LSB bdd in the variable. */
  for(n = 0; v1[n] != 0; n++) ;

  result = malloc((n + 1) * sizeof(int));
  result[n] = 0;

  carry = carryin;
  for(i = n-1; i >= 0; i--) {
    /* result[i] = v1[i] + v2[i] + carry */
    temp[0] = bdd_xor(bddm, v1[i], v2[i]);
    result[i] = bdd_xor(bddm, carry, temp[0]);
    v_free_bdd(temp[0]);
    /* simplify result: allow only values satisfying the context */

    temp[0] = bdd_reduce(bddm, result[i], context);
    v_free_bdd(result[i]);
    result[i] = temp[0];

    /* computes new carry */
    temp[0]  = bdd_and(bddm, v1[i], v2[i]);
    temp[1]  = bdd_or(bddm, v1[i], v2[i]);
    newcarry = bdd_ite(bddm, carry, temp[1], temp[0]);
    v_free_bdd(carry);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
    /* simplify carry: allow only values satisfying the context */
    carry = bdd_reduce(bddm, newcarry, context);
    v_free_bdd(newcarry);
  };

  return(result);
};


/*
 * vneg = -v
 */
intvar vneg(v)
intvar v;
{
  int i;
  bdd temp[2];

  /* assumes v1 and v2 have the same length */

  for(i = 0; v[i] != 0; i++) {
    temp[0] = v[i];
    v[i] = bdd_not(bddm, v[i]);
    v_free_bdd(temp[0]);
  };
  return(v);
};


/*
 * veq = v1 == v2
 */
bdd veq(v1, v2)
intvar v1, v2;
{
  int i;
  bdd result;
  bdd temp[2];

  result = bdd_one(bddm);
  for(i = 0; v1[i] != 0; i++) {
    temp[0] = bdd_xnor(bddm, v1[i], v2[i]);
    temp[1] = result;
    result = bdd_and(bddm, result, temp[0]);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
  };
  return(result);
};


/*
 * vneq = v1 != v2
 */
bdd vneq(v1, v2)
intvar v1, v2;
{
  bdd temp[2];

  temp[0] = veq(v1, v2);
  temp[1] = bdd_not(bddm, temp[0]);
  v_free_bdd(temp[0]);
  return(temp[1]);
};


/*
 * vless = v1 < v2
 */
bdd vless(v1, v2)
intvar v1, v2;
{
  /* v1 < v2 iff
   * - size > 1:
   *   v1[0] < v2[0] or 
   *   v1[0] = v2[0] and vbdd_lt(v1[1..size], v2[1..size])
   * - size = 1:
   *   v1[size] < v2[size].
   *                                             (index 0 is MSB)
   */
  int i, n;
  bdd temp[2];
  bdd result;

  /* assumes v1 and v2 have the same length */

  /* Find LSB bdd in the variable. */
  for(n = 0; v1[n] != 0; n++) ;
  n--;

  /* v1[n] < v2[n] is !v1[n] && v2[n] */
  temp[0] = bdd_not(bddm, v1[n]);
  v_free_bdd(v1[n]);
  result = bdd_and(bddm, temp[0], v2[n]);
  v_free_bdd(temp[0]);
  v_free_bdd(v2[n]);
  for(i = n-1; i >= 0; i--) {
    /* v1[i] < v2[i] || (v1[i] == v2[i] && result) */
    temp[0] = bdd_xnor(bddm, v1[i], v2[i]);
    temp[1] = result;
    result = bdd_and(bddm, temp[0], result);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
    /* v1[i] < v2[i] is !v1[i] && v2[i] */
    temp[0] = bdd_not(bddm, v1[i]);
    temp[1] = bdd_and(bddm, temp[0], v2[i]);
    v_free_bdd(temp[0]);
    temp[0] = result;
    result = bdd_or(bddm, temp[1], result);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
  };
  return(result);
}


/*
 * vleq = v1 <= v2
 */
bdd vleq(v1, v2)
intvar v1, v2;
{
  /* v1 <= v2 iff
   * - size > 1:
   *   v1[0] < v2[0] or 
   *   v1[0] = v2[0] and vbdd_lt(v1[1..size], v2[1..size])
   * - size = 1:
   *   v1[size] <= v2[size].
   *                                             (index 0 is MSB)
   */
  int i, n;
  bdd temp[2];
  bdd result;

  /* assumes v1 and v2 have the same length */

  /* Find LSB bdd in the variable. */
  for(n = 0; v1[n] != 0; n++) ;
  n--;

  /* v1[n] <= v2[n] is !v1[n] || v2[n] */
  temp[0] = bdd_not(bddm, v1[n]);
  result = bdd_or(bddm, temp[0], v2[n]);
  v_free_bdd(temp[0]);
  for(i = n-1; i >= 0; i--) {
    /* v1[i] < v2[i] || (v1[i] == v2[i] && result) */
    temp[0] = bdd_xnor(bddm, v1[i], v2[i]);
    temp[1] = result;
    result = bdd_and(bddm, temp[0], result);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
    /* v1[i] < v2[i] is !v1[i] && v2[i] */
    temp[0] = bdd_not(bddm, v1[i]);
    temp[1] = bdd_and(bddm, temp[0], v2[i]);
    v_free_bdd(temp[0]);
    temp[0] = result;
    result = bdd_or(bddm, temp[1], result);
    v_free_bdd(temp[0]);
    v_free_bdd(temp[1]);
  };
  return(result);
};


/*
 * vgreater = v1 > v2
 */
bdd vgreater(v1, v2)
intvar v1, v2;
{
  bdd temp[2];

  temp[0] = vleq(v1, v2);
  temp[1] = bdd_not(bddm, temp[0]);
  v_free_bdd(temp[0]);
  return(temp[1]);
};


/*
 * vgeq = v1 >= v2
 */
bdd vgeq(v1, v2)
intvar v1, v2;
{
  bdd temp[2];

  temp[0] = vless(v1, v2);
  temp[1] = bdd_not(bddm, temp[0]);
  v_free_bdd(temp[0]);
  return(temp[1]);
};


/*
 * Returns the bdd that corresponds to the integer constant c.
 *
 */
intvar const_to_intvar(c)
int c;
{
  intvar       v;
  int i, n, actual_value;

  /* assumes default size for constant - must change for variable size ints */
  v = malloc((DEFAULT_INT_SIZE+1) * sizeof(int));

  actual_value = 0;
  for(i = DEFAULT_INT_SIZE - 1; i >= 0; i--) {
    if (c & 0x1) {
      v[i] = bdd_one(bddm);
      actual_value += 1 << (n - i);
    } else {
      v[i] = bdd_zero(bddm);
    };

    c >>= 1;
  };
  if (c != 0) {
    error("Integer constant conversion overflow. Value represented is %d.",
                 actual_value);
  };
  v[DEFAULT_INT_SIZE] = 0;

  return(v);
};


intvar v_incref_int(v)
intvar v;
{
  int i;

  for(i = 0; v[i] != 0; i++)
    v[i] = bdd_identity(bddm, v[i]);

  return(v);
};


void v_free_int(v)
intvar v;
{
  int i;

  for(i = 0; v[i] != 0; i++)
    v_free_bdd(v[i]);
};


void v_print_intvar(s, v)
char *s;
intvar v;
{
  int i, n;

  printf("%s\n", s);
  /* Find last bdd in the variable. This will be the LSB */
  for(n = 0; v[n] != 0; n++) ;
  n--;

  for(i = n; i >= 0; i--) {
    printf("v[%3d] = ",n-i);
    v_print_bdd("", v[i]);
  };
};
