
/*
 * The Verus Model Checking System
 * Copyright (c) 1997 Carnegie Mellon University
 * All Rights Reserved. See file license.txt for details.
 *
 *
 * Symbol table management header file
 *
 * Sergio Campos -- 1997
 */

#ifndef SYMBOLS_H
#define SYMBOLS_H

#include <error.h>
#include <bdduser.h>

/* Definitions for the structure that stores symbols */
#define SYMTYPE_EMPTY         0
#define SYMTYPE_FUNCTION      1
#define SYMTYPE_LCL_VARIABLE  2     /* local variable */
#define SYMTYPE_GBL_VARIABLE  3     /* global variable */
#define SYMTYPE_EXT_VARIABLE  4     /* external variable */
#define SYMTYPE_CONSTANT      5
#define SYMTYPE_ASSERT        6

typedef int symbol_type;


#define VARTYPE_BOOLEAN       1
#define VARTYPE_INTEGER       2
#define VARTYPE_INTLIST       3
#define VARTYPE_BOOLIST       4

typedef int variable_type;


typedef struct symbol *symbol_ptr;

/*
 * For variables, value1 is the bdd variable for the next state, and
 * value2 is the bdd for the current state. (The reason is that value2
 * vars are created *before* value1 vars in the bdd ordering)
 */
struct symbol{
  char           *name;     /* pointer to name of symbol in string block */
  symbol_type    symtype;
  variable_type  vartype;
  int            value1; /* next state variable */
  int            value2; /* current state variable */
  int            value3; /* auxiliary var used in substitutions */
  int            value4; /* auxiliary var used for global variables */

  /* Fields used to print symbol value. */
  int            current_value;  /* current value already seen in the bdd */
  int            nobits;         /* number of bits already seen */

  symbol_ptr     next;
};


#define MAX_SYMBOL_SIZE 200
/* used to set variable scope for the symbol table. */
char *function_name;


char *lookup_string();
int insert_string();

vresult init_symbol_table();

/* lookup_symbol checks if an identifier is in the symbol table. */
vresult lookup_symbol();

/* lookup_symbol_fullname checks if an identifier *prepended with the
   current function name* is in the symbol table. */
vresult lookup_symbol_fullname();

symbol_ptr insert_symbol();

vresult init_wait_table();
int find_wc();


/*
 * Global variable table. Used to set up the global identity correctly.
 *
 * For each global variable a bdd global_identity is maintained. This
 * bdd says that the value of that global variable does not change,
 * unless the program is in certain states. For example, global_identity
 * may be (a = a') || (p1._wc == 2), if the only place where a value is
 * assigned to a is the second wait counter of process 1.
 */
typedef struct globalvar *globalvar_ptr;

struct globalvar{
  char           *gname;     /* pointer to name of symbol in string block */
  bdd            assignedv;
  bdd            global_identity; /* next state variable */
  globalvar_ptr     next;
};

extern globalvar_ptr globalvar_table;

vresult insert_globalvar();
globalvar_ptr lookup_globalvar();

#endif
