
/*
 * The Verus Model Checking System
 * Copyright (c) 1997 Carnegie Mellon University
 * All Rights Reserved. See file license.txt for details.
 *
 * Lexical Analyzer for Verus
 *
 * Sergio Campos -- 1997
 */

%{
#include <error.h>
#include <symbols.h>
#include <compile.h>
#include <y.tab.h>

int srcline = 1;

/*
 * YY_INPUT returns the next character in the input buffer. Adapted to
 * use internal buffer instead of input file.
 */
#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)  (result = v_get(buf, max_size))

#undef ECHO
#define ECHO fatal_error("Unknown character in source code.");

%}

%%

[ \t]              ;
\n                 srcline++;

"/*"               { 
                     char c1, c2;
	             /*
                      * When a comment start is encountered, skip
                      * everything until the close of the comment
                      */
                     c1 = c2 = input();
                     if (c1 == '\n') srcline++;
                     while (!(c1 == '*' && c2 == '/')){
                       c1 = c2;
                       c2 = input();
                       if (c2 == '\n') srcline++;
                     }
                   } 


"."                return(DOT);

"+"                return(PLUS);
"-"                return(MINUS);

"=="               return(EQUAL);
"!="               return(NEQUAL);
"<"                return(LESS);
">"                return(GREATER);
"<="               return(LESSEQ);
">="               return(GREATEQ);

"!"                return(NOT);
"&&"               return(AND);
"||"               return(OR);

","                return(COMMA);

":"                return(COLON);
";"                return(SEMICOLON);

"="                return(ASSIGN);
".."               return(DOTDOT);

"("                return(LP);
")"                return(RP);
"["                return(LB);
"]"                return(RB);
"{"                return(LCB);
"}"                return(RCB);

"extern"           return(EXTERN);

"boolean"          return(BOOLEAN);
"int"              return(INTEGER);

"else"             return(ELSE);
"wait"             return(WAIT);

"while"            return(WHILE);
"if"               return(IF);
"select"           return(SELECT);

"var"              return(VAR);

"spec"             return(SPEC);

"->"               return(IMPLIES);
"<->"              return(IFF);
"EX"               return(EX);
"AX"               return(AX);
"EF"               return(EF);
"AF"               return(AF);
"EG"               return(EG);
"AG"               return(AG);
"E"                return(E);
"A"                return(A);
"U"                return(UNTIL);

"MIN"              return(MIN);
"MAX"              return(MAX);
"MINCOUNT"         return(MINCOUNT);
"MAXCOUNT"         return(MAXCOUNT);

"EXAMPLE"          return(EXAMPLE);

  /* BEWARE: Flex (on Linux) doesn't like a newline after the rule and
     before the actions begin. Nor does it like comments that start on
     column 1. Go figure... */
[A-Za-z][A-Za-z0-9._\$-]* {
                     /*
                      * Insert identifier in the string table. This table
                      * is used by the symbol table, and this insertion will
                      * be used directly by those functions.
                      */
                     yylval.string_t = (char *) malloc(strlen(yytext) + 1);
                     strcpy(yylval.string_t, yytext);
                     return(IDENTIFIER);
                   }
[0-9]+             {
                     sscanf(yytext,"%d",&yylval.int_t);
                     return(CONSTANT);
                   }

%%

extern char *filestart;
extern long  filepos;

/*
 * vget returns the next character in the internal input buffer.
 */
int v_get(buf, max_size)
char *buf;
int  max_size;
{
  buf[0] = filestart[filepos];
  filepos++;
  if (buf[0] == EOF) {
    return(YY_NULL);
  };
  return(1);
};


